<?php
/**
 * Copyright © 2016 Magento. All rights reserved.
 * See COPYING.txt for license details.
 */

namespace FiloBlu\Storelocator\Controller\PickUpInStore;

use \Magento\Framework\Exception\NotFoundException;

/**
 * Responsible for loading page content.
 *
 * This is a basic controller that only loads the corresponding layout file. It may duplicate other such
 * controllers, and thus it is considered tech debt. This code duplication will be resolved in future releases.
 */
class RmareceivedAction extends \Magento\Framework\App\Action\Action
{
    protected $jsonResultFactory;
    protected $helper;
    protected $_rma;
    protected $_rmaHistory;
    protected $_transportBuilder;
    protected $_storeManager;
    protected $_inlineTranslation;
    protected $_paymentHelper;
    protected $_addressRenderer;


    public function __construct(
        \Magento\Framework\App\Action\Context $context ,
        \Magento\Framework\Controller\Result\JsonFactory $resultJsonFactory ,
        \FiloBlu\Storelocator\Helper\Data $helper ,
        \Magento\Rma\Model\Rma $rma ,
        \Magento\Rma\Model\Rma\Status\History $rmaHistory ,
        \Magento\Framework\Stdlib\DateTime\DateTime $date ,
        \Magento\Framework\Mail\Template\TransportBuilder $transportBuilder ,
        \Magento\Store\Model\StoreManagerInterface $storeManager ,
        \Magento\Framework\Translate\Inline\StateInterface $inlineTranslation ,
        \Magento\Payment\Helper\Data $paymentHelper ,
        \Magento\Sales\Model\Order\Address\Renderer $addressRenderer ,
        \Magento\Framework\App\ResourceConnection $resourceConnection
    )
    {
        $this->resultJsonFactory = $resultJsonFactory;
        $this->helper = $helper;
        $this->_rma = $rma;
        $this->_rmaHistory = $rmaHistory;
        $this->date = $date;
        $this->_transportBuilder = $transportBuilder;
        $this->_storeManager = $storeManager;
        $this->_inlineTranslation = $inlineTranslation;
        $this->_paymentHelper = $paymentHelper;
        $this->_addressRenderer = $addressRenderer;
        $this->_resourceConnection = $resourceConnection;
        parent::__construct($context);
    }


    public function execute()
    {

        $params = $this->getRequest()->getParams();

        if (!$this->helper->getConfig('carriers/pickupinstore/active') || !isset($params["rma_id"]) || !$params["rma_id"] || !isset($params["hash"]) || !$params["hash"]) {
            return $this->_forward('noRoute');
        }

        $this->_rma->load($params["rma_id"]);

        if ($this->_rma->getId() && $this->_rma->getReturnToStoreHash() == $params["hash"] && (int)$this->_rma->getReturnToStoreId() > 0) {

            if (!$this->helper->getConfig(\FiloBlu\Storelocator\Helper\Data::USE_STANDARD_PICKRMA_MAIL_FLOW)) {
                if ($this->_rma->getStatus() === 'pending') {
                    $status = \Magento\Rma\Model\Item\Attribute\Source\Status::STATE_AUTHORIZED;
                    $this->_rma->setStatus($status);
                }

                /*
                if ($this->_rma->getOrigData('status') === 'authorized' && $this->_rma->getStatus() === 'authorized') {
                    $this->_rma->setReturnToStoreReceivedDate($this->date->gmtDate());
                    $status = \Magento\Rma\Model\Item\Attribute\Source\Status::STATE_RECEIVED;
                    $this->_rma->setStatus($status);
                }*/

            } else {
                $this->_rma->setReturnToStoreReceivedDate($this->date->gmtDate());
            }

            $this->_rma->save();
            $resultJson = $this->resultJsonFactory->create();
            $resultJson->setData(["message" => __("You confirmed that the package ") . $this->_rma->getData("increment_id") . __(" was accepted.")]);
            $this->updateRma();

            if (!$this->helper->getConfig(\FiloBlu\Storelocator\Helper\Data::USE_STANDARD_PICKRMA_MAIL_FLOW)) {
                $statusHistory = $this->_objectManager->create(\Magento\Rma\Model\Rma\Status\History::class);
                $statusHistory->setRmaEntityId($this->_rma->getEntityId());
                if ($this->_rma->getIsSendAuthEmail()) {
                    $statusHistory->sendAuthorizeEmail();
                }
                if ($this->_rma->getStatus() !== $this->_rma->getOrigData('status')) {
                    $this->_rma->saveSystemComment();
                }
            }

            $this->_eventManager->dispatch('filoblu_storelocator_pickupinstore_rma_received' , ['rma' => $this->_rma]);

            return $resultJson;
        } else {
            return $this->_forward('noRoute');
        }
    }

    public function updateRma()
    {

        $connection = $this->_resourceConnection->getConnection();
        $tableName = $this->_resourceConnection->getTableName('magento_rma_item_entity');

        $query_rma_items = "SELECT * FROM " . $tableName . " WHERE rma_entity_id = " . $this->_rma->getData('entity_id');
        $result = $connection->fetchAll($query_rma_items);
        foreach ($result as $idx => $item) {
            $item_id = $item['entity_id'];
            $qty_request = $item['qty_requested'];

            if ((int)$qty_request > 0) {

                $approvedReturned = '';

                if ($this->helper->getConfig(\FiloBlu\Storelocator\Helper\Data::USE_STANDARD_PICKRMA_MAIL_FLOW)) {
                    $approvedReturned = ", qty_approved = " . $qty_request . ", qty_returned = " . $qty_request;
                } else {
                    if ($this->_rma->getOrigData('status') === 'authorized' && $this->_rma->getStatus() === 'received') {
                        $approvedReturned = ", qty_approved = " . $qty_request . ", qty_returned = " . $qty_request;
                    }
                }

                $sql = "UPDATE " . $tableName . " SET status = '" . \Magento\Rma\Model\Item\Attribute\Source\Status::STATE_RECEIVED . "', 
                qty_authorized = " . $qty_request . $approvedReturned . " WHERE entity_id = " . $item_id;

                $connection->query($sql);
            }
        }

        $comment = 'Return In Store - The package was picked up by store';
        $this->_rmaHistory->setRmaEntityId($this->_rma->getId());
        $this->_rmaHistory->setComment($comment);
        $this->_rmaHistory->saveComment($comment , false , true);

    }
}
